@@ -12,9 +12,9 @@ from Cython import Utils
 
 # need one-characters subsitutions (for now) so offsets aren't off
 special_chars = [(u'<', u'\xF0', u'&lt;'),
-                 (u'>', u'\xF1', u'&gt;'), 
+                 (u'>', u'\xF1', u'&gt;'),
                  (u'&', u'\xF2', u'&amp;')]
-                 
+
 line_pos_comment = re.compile(r'/\*.*?<<<<<<<<<<<<<<.*?\*/\n*', re.DOTALL)
 
 class AnnotationCCodeWriter(CCodeWriter):
@@ -32,19 +32,19 @@ class AnnotationCCodeWriter(CCodeWriter):
             self.annotations = create_from.annotations
             self.code = create_from.code
             self.last_pos = create_from.last_pos
-    
+
     def create_new(self, create_from, buffer, copy_formatting):
         return AnnotationCCodeWriter(create_from, buffer, copy_formatting)
 
     def write(self, s):
         CCodeWriter.write(self, s)
         self.annotation_buffer.write(s)
-        
+
     def mark_pos(self, pos):
         if pos is not None:
             CCodeWriter.mark_pos(self, pos)
         if self.last_pos:
-            pos_code = self.code.setdefault(self.last_pos[0].get_description(),{})
+            pos_code = self.code.setdefault(self.last_pos[0].filename,{})
             code = pos_code.get(self.last_pos[1], "")
             pos_code[self.last_pos[1]] = code + self.annotation_buffer.getvalue()
         self.annotation_buffer = StringIO()
@@ -52,7 +52,7 @@ class AnnotationCCodeWriter(CCodeWriter):
 
     def annotate(self, pos, item):
         self.annotations.append((pos, item))
-        
+
     def save_annotation(self, source_filename, target_filename):
         self.mark_pos(None)
         f = Utils.open_source_file(source_filename)
@@ -74,7 +74,7 @@ class AnnotationCCodeWriter(CCodeWriter):
                         all.append(((source_filename, pos[1], pos[2]+size), end))
                     else:
                         all.append((pos, start+end))
-                
+
         all.sort()
         all.reverse()
         for pos, item in all:
@@ -83,9 +83,10 @@ class AnnotationCCodeWriter(CCodeWriter):
             col += 1
             line = lines[line_no]
             lines[line_no] = line[:col] + item + line[col:]
-        
+
         html_filename = os.path.splitext(target_filename)[0] + ".html"
         f = codecs.open(html_filename, "w", encoding="UTF-8")
+        f.write(u'<!-- Generated by Cython %s on %s -->\n' % (Version.version, time.asctime()))
         f.write(u'<html>\n')
         f.write(u"""
 <head>
@@ -130,14 +131,14 @@ function toggleDiv(id) {
         c_file = Utils.decode_filename(os.path.basename(target_filename))
         f.write(u'<p>Raw output: <a href="%s">%s</a>\n' % (c_file, c_file))
         k = 0
-        
+
         py_c_api = re.compile(u'(Py[A-Z][a-z]+_[A-Z][a-z][A-Za-z_]+)\(')
         py_marco_api = re.compile(u'(Py[A-Z][a-z]+_[A-Z][A-Z_]+)\(')
         pyx_c_api = re.compile(u'(__Pyx_[A-Z][a-z_][A-Za-z_]+)\(')
         pyx_macro_api = re.compile(u'(__Pyx_[A-Z][A-Z_]+)\(')
         error_goto = re.compile(ur'((; *if .*)? \{__pyx_filename = .*goto __pyx_L\w+;\})')
         refnanny = re.compile(u'(__Pyx_X?(GOT|GIVE)REF|__Pyx_RefNanny[A-Za-z]+)')
-        
+
         code_source_file = self.code[source_filename]
         for line in lines:
 
@@ -146,18 +147,18 @@ function toggleDiv(id) {
                 code = code_source_file[k]
             except KeyError:
                 code = ''
-            
+
             code = code.replace('<', '<code><</code>')
-            
+
             code, py_c_api_calls = py_c_api.subn(ur"<span class='py_c_api'>\1</span>(", code)
             code, pyx_c_api_calls = pyx_c_api.subn(ur"<span class='pyx_c_api'>\1</span>(", code)
             code, py_macro_api_calls = py_marco_api.subn(ur"<span class='py_macro_api'>\1</span>(", code)
             code, pyx_macro_api_calls = pyx_macro_api.subn(ur"<span class='pyx_macro_api'>\1</span>(", code)
             code, refnanny_calls = refnanny.subn(ur"<span class='refnanny'>\1</span>", code)
             code, error_goto_calls = error_goto.subn(ur"<span class='error_goto'>\1</span>", code)
-            
+
             code = code.replace(u"<span class='error_goto'>;", u";<span class='error_goto'>")
-            
+
             score = 5*py_c_api_calls + 2*pyx_c_api_calls + py_macro_api_calls + pyx_macro_api_calls - refnanny_calls
             color = u"FFFF%02x" % int(255/(1+score/10.0))
             f.write(u"<pre class='line' style='background-color: #%s' onclick='toggleDiv(\"line%s\")'>" % (color, k))
@@ -166,13 +167,13 @@ function toggleDiv(id) {
             for c, cc, html in special_chars:
                 line = line.replace(cc, html)
             f.write(line.rstrip())
-                
+
             f.write(u'</pre>\n')
             code = re.sub(line_pos_comment, '', code) # inline annotations are redundant
             f.write(u"<pre id='line%s' class='code' style='background-color: #%s'>%s</pre>" % (k, color, code))
         f.write(u'</body></html>\n')
         f.close()
-        
+
 
 # TODO: make this cleaner
 def escape(raw_string):
@@ -184,15 +185,15 @@ def escape(raw_string):
 
 
 class AnnotationItem(object):
-    
+
     def __init__(self, style, text, tag="", size=0):
         self.style = style
         self.text = text
         self.tag = tag
         self.size = size
-        
+
     def start(self):
         return u"<span class='tag %s' title='%s'>%s" % (self.style, self.text, self.tag)
-    
+
     def end(self):
         return self.size, u"</span>"
