@@ -2,50 +2,70 @@
 #  Cython - Compilation-wide options and pragma declarations
 #
 
-cache_builtins = 1  #  Perform lookups on builtin names only once
+# Perform lookups on builtin names only once, at module initialisation
+# time.  This will prevent the module from getting imported if a
+# builtin name that it uses cannot be found during initialisation.
+cache_builtins = True
 
-embed_pos_in_docstring = 0
-gcc_branch_hints = 1
+embed_pos_in_docstring = False
+gcc_branch_hints = True
 
 pre_import = None
 docstrings = True
 
-# Decref global variables in this module on exit for garbage collection. 
+# Decref global variables in this module on exit for garbage collection.
 # 0: None, 1+: interned objects, 2+: cdef globals, 3+: types objects
 # Mostly for reducing noise for Valgrind, only executes at process exit
-# (when all memory will be reclaimed anyways). 
-generate_cleanup_code = 0
+# (when all memory will be reclaimed anyways).
+generate_cleanup_code = False
 
-annotate = 0
+annotate = False
 
-# This will convert statements of the form "for i in range(...)" 
+# This will abort the compilation on the first error occured rather than trying
+# to keep going and printing further error messages.
+fast_fail = False
+
+# Make all warnings into errors.
+warning_errors = False
+
+# Make unknown names an error.  Python raises a NameError when
+# encountering unknown names at runtime, whereas this option makes
+# them a compile time error.  If you want full Python compatibility,
+# you should disable this option and also 'cache_builtins'.
+error_on_unknown_names = True
+
+# This will convert statements of the form "for i in range(...)"
 # to "for i from ..." when i is a cdef'd integer type, and the direction
-# (i.e. sign of step) can be determined. 
-# WARNING: This may change the semantics if the range causes assignment to 
+# (i.e. sign of step) can be determined.
+# WARNING: This may change the semantics if the range causes assignment to
 # i to overflow. Specifically, if this option is set, an error will be
 # raised before the loop is entered, wheras without this option the loop
-# will execute until an overflowing value is encountered. 
-convert_range = 1
+# will execute until an overflowing value is encountered.
+convert_range = True
 
-# Enable this to allow one to write your_module.foo = ... to overwrite the 
-# definition if the cpdef function foo, at the cost of an extra dictionary 
-# lookup on every call. 
-# If this is 0 it simply creates a wrapper. 
-lookup_module_cpdef = 0
+# Enable this to allow one to write your_module.foo = ... to overwrite the
+# definition if the cpdef function foo, at the cost of an extra dictionary
+# lookup on every call.
+# If this is 0 it simply creates a wrapper.
+lookup_module_cpdef = False
 
-# This will set local variables to None rather than NULL which may cause 
-# surpress what would be an UnboundLocalError in pure Python but eliminates 
-# checking for NULL on every use, and can decref rather than xdecref at the end. 
-# WARNING: This is a work in progress, may currently segfault.
-init_local_none = 1
+# Whether or not to embed the Python interpreter, for use in making a
+# standalone executable or calling from external libraries.
+# This will provide a method which initalizes the interpreter and
+# executes the body of this module.
+embed = None
 
-# Append the c file and line number to the traceback for exceptions. 
-c_line_in_traceback = 1
+# Disables function redefinition, allowing all functions to be declared at
+# module creation time. For legacy code only, needed for some circular imports.
+disable_function_redefinition = False
+
+# In previous iterations of Cython, globals() gave the first non-Cython module
+# globals in the call stack.  Sage relies on this behavior for variable injection.
+old_style_globals = False
+
+# Allows cimporting from a pyx file without a pxd file.
+cimport_from_pyx = False
 
-# Whether or not to embed the Python interpreter, for use in making a 
-# standalone executable. This will provide a main() method which simply 
-# executes the body of this module. 
-embed = False
 
 
 # Declare compiler directives
@@ -62,14 +82,33 @@ directive_defaults = {
     'wraparound' : True,
     'ccomplex' : False, # use C99/C++ for complex types and arith
     'callspec' : "",
+    'final' : False,
+    'internal' : False,
     'profile': False,
     'infer_types': None,
     'infer_types.verbose': False,
     'autotestdict': True,
+    'autotestdict.cdef': False,
+    'autotestdict.all': False,
     'language_level': 2,
-    
+    'fast_getattr': False, # Undocumented until we come up with a better way to handle this everywhere.
+    'py2_import': False, # For backward compatibility of Cython's source code
+
     'warn': None,
     'warn.undeclared': False,
+    'warn.unreachable': True,
+    'warn.maybe_uninitialized': False,
+    'warn.unreachable': True,
+    'warn.unused': False,
+    'warn.unused_arg': False,
+    'warn.unused_result': False,
+
+# remove unreachable code
+    'remove_unreachable': True,
+
+# control flow debug directives
+    'control_flow.dot_output': "", # Graphviz output filename
+    'control_flow.dot_annotate_defs': False, # Annotate definitions
 
 # test support
     'test_assert_path_exists' : [],
@@ -79,9 +118,21 @@ directive_defaults = {
     'binding': False,
 }
 
+# Extra warning directives
+extra_warnings = {
+    'warn.maybe_uninitialized': True,
+    'warn.unreachable': True,
+    'warn.unused': True,
+}
+
 # Override types possibilities above, if needed
 directive_types = {
+    'final' : bool,  # final cdef classes and methods
+    'internal' : bool,  # cdef class visibility in the module dict
     'infer_types' : bool, # values can be True/None/False
+    'cfunc' : None, # decorators do not take directive value
+    'ccall' : None,
+    'cclass' : None,
     }
 
 for key, val in directive_defaults.items():
@@ -90,9 +141,13 @@ for key, val in directive_defaults.items():
 
 directive_scopes = { # defaults to available everywhere
     # 'module', 'function', 'class', 'with statement'
+    'final' : ('cclass',),   # add 'method' in the future
+    'internal' : ('cclass',),
     'autotestdict' : ('module',),
-    'test_assert_path_exists' : ('function',),
-    'test_fail_if_path_exists' : ('function',),
+    'autotestdict.all' : ('module',),
+    'autotestdict.cdef' : ('module',),
+    'test_assert_path_exists' : ('function', 'class', 'cclass'),
+    'test_fail_if_path_exists' : ('function', 'class', 'cclass'),
 }
 
 def parse_directive_value(name, value, relaxed_bool=False):
@@ -108,7 +163,7 @@ def parse_directive_value(name, value, relaxed_bool=False):
     Traceback (most recent call last):
        ...
     ValueError: boundscheck directive must be set to True or False, got 'true'
-    
+
     """
     type = directive_types.get(name)
     if not type: return None
