@@ -1,3 +1,4 @@
+# cython: infer_types=True, language_level=3, py2_import=True
 #
 #   Cython Scanner
 #
@@ -7,7 +8,7 @@ import os
 import platform
 
 import cython
-cython.declare(EncodedString=object, string_prefixes=object, raw_prefixes=object, IDENT=object,
+cython.declare(EncodedString=object, string_prefixes=object, raw_prefixes=object, IDENT=unicode,
                print_function=object)
 
 from Cython import Plex, Utils
@@ -31,11 +32,11 @@ def get_lexicon():
     if not lexicon:
         lexicon = make_lexicon()
     return lexicon
-    
+
 #------------------------------------------------------------------
 
 py_reserved_words = [
-    "global", "def", "class", "print", "del", "pass", "break",
+    "global", "nonlocal", "def", "class", "print", "del", "pass", "break",
     "continue", "return", "raise", "import", "exec", "try",
     "except", "finally", "while", "if", "elif", "else", "for",
     "in", "assert", "and", "or", "not", "is", "in", "lambda",
@@ -44,7 +45,7 @@ py_reserved_words = [
 
 pyx_reserved_words = py_reserved_words + [
     "include", "ctypedef", "cdef", "cpdef",
-    "cimport", "by", "DEF", "IF", "ELIF", "ELSE"
+    "cimport", "DEF", "IF", "ELIF", "ELSE"
 ]
 
 class Method(object):
@@ -52,7 +53,7 @@ class Method(object):
     def __init__(self, name):
         self.name = name
         self.__name__ = name # for Plex tracing
-    
+
     def __call__(self, stream, text):
         return getattr(stream, self.name)(text)
 
@@ -63,16 +64,16 @@ class CompileTimeScope(object):
     def __init__(self, outer = None):
         self.entries = {}
         self.outer = outer
-    
+
     def declare(self, name, value):
         self.entries[name] = value
-    
+
     def lookup_here(self, name):
         return self.entries[name]
-        
+
     def __contains__(self, name):
         return name in self.entries
-    
+
     def lookup(self, name):
         try:
             return self.lookup_here(name)
@@ -165,25 +166,54 @@ class FileSourceDescriptor(SourceDescriptor):
     optional name argument and will be passed back when asking for
     the position()-tuple.
     """
-    def __init__(self, filename):
+    def __init__(self, filename, path_description=None):
         filename = Utils.decode_filename(filename)
+        self.path_description = path_description or filename
         self.filename = filename
         self.set_file_type_from_name(filename)
         self._cmp_name = filename
-    
+        self._lines = {}
+
     def get_lines(self, encoding=None, error_handling=None):
-        return Utils.open_source_file(
+        # we cache the lines only the second time this is called, in
+        # order to save memory when they are only used once
+        key = (encoding, error_handling)
+        try:
+            lines = self._lines[key]
+            if lines is not None:
+                return lines
+        except KeyError:
+            pass
+        f = Utils.open_source_file(
             self.filename, encoding=encoding,
             error_handling=error_handling,
             # newline normalisation is costly before Py2.6
             require_normalised_newlines=False)
-    
+        try:
+            lines = list(f)
+        finally:
+            f.close()
+        if key in self._lines:
+            self._lines[key] = lines
+        else:
+            # do not cache the first access, but remember that we
+            # already read it once
+            self._lines[key] = None
+        return lines
+
     def get_description(self):
-        return self.filename
-    
+        return self.path_description
+
+    def get_error_description(self):
+        path = self.filename
+        cwd = Utils.decode_filename(os.getcwd() + os.path.sep)
+        if path.startswith(cwd):
+            return path[len(cwd):]
+        return path
+
     def get_filenametable_entry(self):
         return self.filename
-    
+
     def __eq__(self, other):
         return isinstance(other, FileSourceDescriptor) and self.filename == other.filename
 
@@ -198,12 +228,14 @@ class StringSourceDescriptor(SourceDescriptor):
     Instances of this class can be used instead of a filenames if the
     code originates from a string object.
     """
+    filename = None
+
     def __init__(self, name, code):
         self.name = name
         #self.set_file_type_from_name(name)
         self.codelines = [x + "\n" for x in code.split("\n")]
         self._cmp_name = name
-    
+
     def get_lines(self, encoding=None, error_handling=None):
         if not encoding:
             return self.codelines
@@ -214,11 +246,16 @@ class StringSourceDescriptor(SourceDescriptor):
     def get_description(self):
         return self.name
 
+    get_error_description = get_description
+
     def get_filenametable_entry(self):
         return "stringsource"
 
     def __hash__(self):
-        return hash(self.name)
+        return id(self)
+        # Do not hash on the name, an identical string source should be the
+        # same object (name is often defaulted in other places)
+        # return hash(self.name)
 
     def __eq__(self, other):
         return isinstance(other, StringSourceDescriptor) and self.name == other.name
@@ -235,7 +272,7 @@ class PyrexScanner(Scanner):
     #  compile_time_eval  boolean  In a true conditional compilation context
     #  compile_time_expr  boolean  In a compile-time expression context
 
-    def __init__(self, file, filename, parent_scanner = None, 
+    def __init__(self, file, filename, parent_scanner = None,
                  scope = None, context = None, source_encoding=None, parse_comments=True, initial_pos=None):
         Scanner.__init__(self, get_lexicon(), file, filename, initial_pos)
         if parent_scanner:
@@ -268,8 +305,8 @@ class PyrexScanner(Scanner):
 
     def commentline(self, text):
         if self.parse_comments:
-            self.produce('commentline', text)    
-    
+            self.produce('commentline', text)
+
     def current_level(self):
         return self.indentation_stack[-1]
 
@@ -285,14 +322,14 @@ class PyrexScanner(Scanner):
         if self.bracket_nesting_level == 0:
             self.begin('INDENT')
             self.produce('NEWLINE', '')
-    
+
     string_states = {
         "'":   'SQ_STRING',
         '"':   'DQ_STRING',
         "'''": 'TSQ_STRING',
         '"""': 'TDQ_STRING'
     }
-    
+
     def begin_string_action(self, text):
         if text[:1] in string_prefixes:
             text = text[1:]
@@ -300,11 +337,11 @@ class PyrexScanner(Scanner):
             text = text[1:]
         self.begin(self.string_states[text])
         self.produce('BEGIN_STRING')
-    
+
     def end_string_action(self, text):
         self.begin('')
         self.produce('END_STRING')
-    
+
     def unclosed_string_action(self, text):
         self.end_string_action(text)
         self.error("Unclosed string literal")
@@ -358,8 +395,11 @@ class PyrexScanner(Scanner):
             self.error("Unrecognized character")
         if sy == IDENT:
             if systring in self.keywords:
-                if systring == 'print' and \
-                       print_function in self.context.future_directives:
+                if systring == u'print' and print_function in self.context.future_directives:
+                    self.keywords.discard('print')
+                    systring = EncodedString(systring)
+                elif systring == u'exec' and self.context.language_level >= 3:
+                    self.keywords.discard('exec')
                     systring = EncodedString(systring)
                 else:
                     sy = systring
@@ -374,7 +414,7 @@ class PyrexScanner(Scanner):
             else:
                 t = "%s %s" % (self.sy, self.systring)
             print("--- %3d %2d %s" % (line, col, t))
-    
+
     def peek(self):
         saved = self.sy, self.systring
         self.next()
@@ -382,16 +422,16 @@ class PyrexScanner(Scanner):
         self.unread(*next)
         self.sy, self.systring = saved
         return next
-    
+
     def put_back(self, sy, systring):
         self.unread(self.sy, self.systring)
         self.sy = sy
         self.systring = systring
-    
+
     def unread(self, token, value):
         # This method should be added to Plex
         self.queue.insert(0, (token, value))
-    
+
     def error(self, message, pos = None, fatal = True):
         if pos is None:
             pos = self.position()
@@ -399,25 +439,29 @@ class PyrexScanner(Scanner):
             err = error(pos, "Possible inconsistent indentation")
         err = error(pos, message)
         if fatal: raise err
-        
+
     def expect(self, what, message = None):
         if self.sy == what:
             self.next()
         else:
             self.expected(what, message)
-    
+
     def expect_keyword(self, what, message = None):
         if self.sy == IDENT and self.systring == what:
             self.next()
         else:
             self.expected(what, message)
-    
+
     def expected(self, what, message = None):
         if message:
             self.error(message)
         else:
-            self.error("Expected '%s'" % what)
-        
+            if self.sy == IDENT:
+                found = self.systring
+            else:
+                found = self.sy
+            self.error("Expected '%s', found '%s'" % (what, found))
+
     def expect_indent(self):
         self.expect('INDENT',
             "Expected an increase in indentation level")
