@@ -1,6 +1,9 @@
 # Run as:
 #    python setup.py build_ext --inplace
 
+import sys
+sys.path.insert(0, "..")
+
 from distutils.core import setup
 from distutils.extension import Extension
 from Cython.Build import cythonize
