@@ -2,6 +2,8 @@
 #define POST_UN_OP(op) const char* operator op (int x) { return "post "#op; }
 #define BIN_OP(op) const char* operator op (int x) { return "binary "#op; }
 
+#define COMMA ,
+
 class TestOps {
 
 public:
@@ -12,12 +14,12 @@ public:
     UN_OP(~);
     UN_OP(!);
     UN_OP(&);
-    
+
     UN_OP(++);
     UN_OP(--);
     POST_UN_OP(++);
     POST_UN_OP(--);
-    
+
     BIN_OP(+);
     BIN_OP(-);
     BIN_OP(*);
@@ -26,10 +28,11 @@ public:
 
     BIN_OP(<<);
     BIN_OP(>>);
-    
+
     BIN_OP(|);
     BIN_OP(&);
     BIN_OP(^);
+    BIN_OP(COMMA);
 
     BIN_OP(==);
     BIN_OP(!=);
